@@ -10,11 +10,13 @@
 # as part of this package.
 
 """Unit tests for the Bio.PDB module."""
+from __future__ import print_function
+
 import os
 import tempfile
 import unittest
 import warnings
-from StringIO import StringIO
+from Bio._py3k import StringIO
 
 try:
     import numpy
@@ -26,6 +28,7 @@ except ImportError:
     raise MissingPythonDependencyError(
         "Install NumPy if you want to use Bio.PDB.")
 
+from Bio import BiopythonWarning
 from Bio.Seq import Seq
 from Bio.Alphabet import generic_protein
 from Bio.PDB import PDBParser, PPBuilder, CaPPBuilder, PDBIO, Select
@@ -34,6 +37,7 @@ from Bio.PDB.PDBExceptions import PDBConstructionException, PDBConstructionWarni
 from Bio.PDB import rotmat, Vector
 from Bio.PDB import Residue, Atom
 from Bio.PDB import make_dssp_dict
+from Bio.PDB.NACCESS import process_asa_data, process_rsa_data
 
 
 # NB: the 'A_' prefix ensures this test case is run first
@@ -48,6 +52,8 @@ class A_ExceptionTest(unittest.TestCase):
 
         NB: The try/finally block is adapted from the warnings.catch_warnings
         context manager in the Python 2.6 standard library.
+
+        TODO: Now we require Python 2.6, switch to using warnings.catch_warnings
         """
         warnings.simplefilter('always', PDBConstructionWarning)
         try:
@@ -86,13 +92,12 @@ class A_ExceptionTest(unittest.TestCase):
 
     def test_2_strict(self):
         """Check error: Parse a flawed PDB file in strict mode."""
-        warnings.simplefilter('ignore', PDBConstructionWarning)
-        try:
-            parser = PDBParser(PERMISSIVE=False)
+        parser = PDBParser(PERMISSIVE=False)
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always", PDBConstructionWarning)
             self.assertRaises(PDBConstructionException,
-                   parser.get_structure, "example", "PDB/a_structure.pdb")
-        finally:
-            warnings.filters.pop()
+                              parser.get_structure, "example", "PDB/a_structure.pdb")
+            self.assertEqual(len(w), 4, w)
 
     def test_3_bad_xyz(self):
         """Check error: Parse an entry with bad x,y,z value."""
@@ -106,7 +111,10 @@ class A_ExceptionTest(unittest.TestCase):
     def test_4_occupancy(self):
         """Parse file with missing occupancy"""
         permissive = PDBParser(PERMISSIVE=True)
-        structure = permissive.get_structure("test", "PDB/occupancy.pdb")
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always", PDBConstructionWarning)
+            structure = permissive.get_structure("test", "PDB/occupancy.pdb")
+            self.assertEqual(len(w), 3, w)
         atoms = structure[0]['A'][(' ', 152, ' ')]
         # Blank occupancy behavior set in Bio/PDB/PDBParser
         self.assertEqual(atoms['N'].get_occupancy(), None)
@@ -138,7 +146,7 @@ class HeaderTests(unittest.TestCase):
                 'release_date': '1998-10-14',
                 'structure_method': 'x-ray diffraction',
                 }
-        for key, expect in known_strings.iteritems():
+        for key, expect in known_strings.items():
             self.assertEqual(struct.header[key].lower(), expect.lower())
 
     def test_fibril(self):
@@ -156,16 +164,16 @@ class HeaderTests(unittest.TestCase):
                 'release_date': '2005-11-22',
                 'structure_method': 'solution nmr',
                 }
-        for key, expect in known_strings.iteritems():
+        for key, expect in known_strings.items():
             self.assertEqual(struct.header[key].lower(), expect.lower())
 
 
 class ParseTest(unittest.TestCase):
     def setUp(self):
-        warnings.simplefilter('ignore', PDBConstructionWarning)
-        p = PDBParser(PERMISSIVE=1)
-        self.structure = p.get_structure("example", "PDB/a_structure.pdb")
-        warnings.filters.pop()
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore", PDBConstructionWarning)
+            p = PDBParser(PERMISSIVE=1)
+            self.structure = p.get_structure("example", "PDB/a_structure.pdb")
 
     def test_c_n(self):
         """Extract polypeptides using C-N."""
@@ -698,10 +706,10 @@ class ParseReal(unittest.TestCase):
 
 class WriteTest(unittest.TestCase):
     def setUp(self):
-        warnings.simplefilter('ignore', PDBConstructionWarning)
-        self.parser = PDBParser(PERMISSIVE=1)
-        self.structure = self.parser.get_structure("example", "PDB/1A8O.pdb")
-        warnings.filters.pop()
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore", PDBConstructionWarning)
+            self.parser = PDBParser(PERMISSIVE=1)
+            self.structure = self.parser.get_structure("example", "PDB/1A8O.pdb")
 
     def test_pdbio_write_structure(self):
         """Write a full structure using PDBIO"""
@@ -788,32 +796,34 @@ class WriteTest(unittest.TestCase):
 
     def test_pdbio_missing_occupancy(self):
         """Write PDB file with missing occupancy"""
-
-        from Bio import BiopythonWarning
-        warnings.simplefilter('ignore', BiopythonWarning)
-
         io = PDBIO()
-        structure = self.parser.get_structure("test", "PDB/occupancy.pdb")
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore", PDBConstructionWarning)
+            structure = self.parser.get_structure("test", "PDB/occupancy.pdb")
         io.set_structure(structure)
         filenumber, filename = tempfile.mkstemp()
         os.close(filenumber)
         try:
-            io.save(filename)
-            struct2 = self.parser.get_structure("test", filename)
+            with warnings.catch_warnings(record=True) as w:
+                warnings.simplefilter("always", BiopythonWarning)
+                io.save(filename)
+                self.assertEqual(len(w), 1, w)
+            with warnings.catch_warnings():
+                warnings.simplefilter("ignore", PDBConstructionWarning)
+                struct2 = self.parser.get_structure("test", filename)
             atoms = struct2[0]['A'][(' ', 152, ' ')]
             self.assertEqual(atoms['N'].get_occupancy(), None)
         finally:
             os.remove(filename)
-            warnings.filters.pop()
 
 
 class Exposure(unittest.TestCase):
     "Testing Bio.PDB.HSExposure."
     def setUp(self):
-        warnings.simplefilter('ignore', PDBConstructionWarning)
         pdb_filename = "PDB/a_structure.pdb"
-        structure=PDBParser(PERMISSIVE=True).get_structure('X', pdb_filename)
-        warnings.filters.pop()
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore", PDBConstructionWarning)
+            structure=PDBParser(PERMISSIVE=True).get_structure('X', pdb_filename)
         self.model=structure[1]
         #Look at first chain only
         a_residues=list(self.model["A"].child_list)
@@ -894,10 +904,10 @@ class Atom_Element(unittest.TestCase):
     """induces Atom Element from Atom Name"""
 
     def setUp(self):
-        warnings.simplefilter('ignore', PDBConstructionWarning)
         pdb_filename = "PDB/a_structure.pdb"
-        structure=PDBParser(PERMISSIVE=True).get_structure('X', pdb_filename)
-        warnings.filters.pop()
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore", PDBConstructionWarning)
+            structure=PDBParser(PERMISSIVE=True).get_structure('X', pdb_filename)
         self.residue = structure[0]['A'][('H_PCA', 1, ' ')]
 
     def test_AtomElement(self):
@@ -936,9 +946,11 @@ class Atom_Element(unittest.TestCase):
             N=(' NH1', ' NH2'),
         )
 
-        for element, atom_names in pdb_elements.iteritems():
+        for element, atom_names in pdb_elements.items():
             for fullname in atom_names:
-                e = quick_assign(fullname)
+                with warnings.catch_warnings():
+                    warnings.simplefilter("ignore", PDBConstructionWarning)
+                    e = quick_assign(fullname)
                 #warnings.warn("%s %s" % (fullname, e))
                 self.assertEqual(e, element)
 
@@ -951,7 +963,7 @@ class IterationTests(unittest.TestCase):
     def test_get_chains(self):
         """Yields chains from different models separately."""
         chains = [chain.id for chain in self.struc.get_chains()]
-        self.assertEqual(chains, ['A','A', 'B', ' '])
+        self.assertEqual(chains, ['A', 'A', 'B', ' '])
 
     def test_get_residues(self):
         """Yields all residues from all models."""
@@ -968,16 +980,14 @@ class IterationTests(unittest.TestCase):
 #    """Tests renumbering of structures."""
 #
 #    def setUp(self):
-#        warnings.simplefilter('ignore', PDBConstructionWarning)
 #        pdb_filename = "PDB/1A8O.pdb"
 #        self.structure=PDBParser(PERMISSIVE=True).get_structure('X', pdb_filename)
-#        warnings.filters.pop()
 #
 #    def test_renumber_residues(self):
 #        """Residues in a structure are renumbered."""
 #        self.structure.renumber_residues()
 #        nums = [resi.id[1] for resi in self.structure[0]['A'].child_list]
-#        print nums
+#        print(nums)
 #
 # -------------------------------------------------------------
 
@@ -999,7 +1009,7 @@ class TransformTests(unittest.TestCase):
         """
         if hasattr(o, "get_coord"):
             return o.get_coord(), 1
-        total_pos = numpy.array((0.0,0.0,0.0))
+        total_pos = numpy.array((0.0, 0.0, 0.0))
         total_count = 0
         for p in o.get_list():
             pos, count = self.get_total_pos(p)
@@ -1017,8 +1027,8 @@ class TransformTests(unittest.TestCase):
     def test_transform(self):
         """Transform entities (rotation and translation)."""
         for o in (self.s, self.m, self.c, self.r, self.a):
-            rotation = rotmat(Vector(1,3,5), Vector(1,0,0))
-            translation=numpy.array((2.4,0,1), 'f')
+            rotation = rotmat(Vector(1, 3, 5), Vector(1, 0, 0))
+            translation=numpy.array((2.4, 0, 1), 'f')
             oldpos = self.get_pos(o)
             o.transform(rotation, translation)
             newpos = self.get_pos(o)
@@ -1060,6 +1070,28 @@ class DsspTests(unittest.TestCase):
         dssp, keys = make_dssp_dict("PDB/2BEG.dssp")
         self.assertEqual(len(dssp), 130)
 
+    def test_DSSP_noheader_file(self):
+        """Test parsing of pregenerated DSSP missing header information"""
+        # New DSSP prints a line containing only whitespace and "."
+        dssp, keys = make_dssp_dict("PDB/2BEG_noheader.dssp")
+        self.assertEqual(len(dssp), 130)
+
+class NACCESSTests(unittest.TestCase):
+    """Tests for NACCESS parsing etc which don't need the binary tool.
+
+    See also test_NACCESS_tool.py for run time testing with the tool.
+    """
+    def test_NACCESS_rsa_file(self):
+        """Test parsing of pregenerated rsa NACCESS file"""
+        with open("PDB/1A8O.rsa") as rsa:
+            naccess = process_rsa_data(rsa)
+        self.assertEqual(len(naccess), 66)
+
+    def test_NACCESS_asa_file(self):
+        """Test parsing of pregenerated asa NACCESS file"""
+        with open("PDB/1A8O.asa") as asa:
+            naccess = process_asa_data(asa)
+        self.assertEqual(len(naccess), 524)
 
 if __name__ == '__main__':
     runner = unittest.TextTestRunner(verbosity=2)
