@@ -5,20 +5,18 @@
 
 """Additional unit tests for Bio.SeqIO.QualityIO (covering FASTQ and QUAL)."""
 
-from __future__ import with_statement
+from __future__ import print_function
 
 import os
 import unittest
 import warnings
 
-from StringIO import StringIO
-try:
-    #This is in Python 2.6+, but we need it on Python 3
-    from io import BytesIO
-except ImportError:
-    BytesIO = StringIO
+from Bio._py3k import range
+from Bio._py3k import StringIO
+from Bio._py3k import _universal_read_mode
+from io import BytesIO
 
-from Bio import BiopythonWarning
+from Bio import BiopythonWarning, BiopythonParserWarning
 from Bio.Alphabet import generic_dna
 from Bio.SeqIO import QualityIO
 from Bio import SeqIO
@@ -55,7 +53,7 @@ def write_read(filename, in_format, out_format):
     SeqIO.write(records, handle, out_format)
     handle.seek(0)
     #Now load it back and check it agrees,
-    records2 = list(SeqIO.parse(handle,out_format))
+    records2 = list(SeqIO.parse(handle, out_format))
     compare_records(records, records2, truncation_expected(out_format))
 
 
@@ -82,16 +80,16 @@ def compare_record(old, new, truncate=None):
     if "phred_quality" in old.letter_annotations \
     and "phred_quality" in new.letter_annotations \
     and old.letter_annotations["phred_quality"] != new.letter_annotations["phred_quality"]:
-        if truncate and [min(q,truncate) for q in old.letter_annotations["phred_quality"]] == \
-                        [min(q,truncate) for q in new.letter_annotations["phred_quality"]]:
+        if truncate and [min(q, truncate) for q in old.letter_annotations["phred_quality"]] == \
+                        [min(q, truncate) for q in new.letter_annotations["phred_quality"]]:
             pass
         else:
             raise ValuerError("Mismatch in phred_quality")
     if "solexa_quality" in old.letter_annotations \
     and "solexa_quality" in new.letter_annotations \
     and old.letter_annotations["solexa_quality"] != new.letter_annotations["solexa_quality"]:
-        if truncate and [min(q,truncate) for q in old.letter_annotations["solexa_quality"]] == \
-                        [min(q,truncate) for q in new.letter_annotations["solexa_quality"]]:
+        if truncate and [min(q, truncate) for q in old.letter_annotations["solexa_quality"]] == \
+                        [min(q, truncate) for q in new.letter_annotations["solexa_quality"]]:
             pass
         else:
             raise ValueError("Mismatch in phred_quality")
@@ -102,12 +100,12 @@ def compare_record(old, new, truncate=None):
         converted = [round(QualityIO.solexa_quality_from_phred(q))
                      for q in old.letter_annotations["phred_quality"]]
         if truncate:
-            converted = [min(q,truncate) for q in converted]
+            converted = [min(q, truncate) for q in converted]
         if converted != new.letter_annotations["solexa_quality"]:
-            print
-            print old.letter_annotations["phred_quality"]
-            print converted
-            print new.letter_annotations["solexa_quality"]
+            print("")
+            print(old.letter_annotations["phred_quality"])
+            print(converted)
+            print(new.letter_annotations["solexa_quality"])
             raise ValueError("Mismatch in phred_quality vs solexa_quality")
     if "solexa_quality" in old.letter_annotations \
     and "phred_quality" in new.letter_annotations:
@@ -116,11 +114,11 @@ def compare_record(old, new, truncate=None):
         converted = [round(QualityIO.phred_quality_from_solexa(q))
                      for q in old.letter_annotations["solexa_quality"]]
         if truncate:
-            converted = [min(q,truncate) for q in converted]
+            converted = [min(q, truncate) for q in converted]
         if converted != new.letter_annotations["phred_quality"]:
-            print old.letter_annotations["solexa_quality"]
-            print converted
-            print new.letter_annotations["phred_quality"]
+            print(old.letter_annotations["solexa_quality"])
+            print(converted)
+            print(new.letter_annotations["phred_quality"])
             raise ValueError("Mismatch in solexa_quality vs phred_quality")
     return True
 
@@ -130,7 +128,7 @@ def compare_records(old_list, new_list, truncate_qual=None):
     if len(old_list) != len(new_list):
         raise ValueError("%i vs %i records" % (len(old_list), len(new_list)))
     for old, new in zip(old_list, new_list):
-        if not compare_record(old,new,truncate_qual):
+        if not compare_record(old, new, truncate_qual):
             return False
     return True
 
@@ -141,24 +139,24 @@ class TestFastqErrors(unittest.TestCase):
         if not formats:
             formats = ["fastq-sanger", "fastq-solexa", "fastq-illumina"]
         for format in formats:
-            handle = open(filename, "rU")
+            handle = open(filename, _universal_read_mode)
             records = SeqIO.parse(handle, format)
             for i in range(good_count):
-                record = records.next()  # Make sure no errors!
+                record = next(records)  # Make sure no errors!
                 self.assertTrue(isinstance(record, SeqRecord))
-            self.assertRaises(ValueError, records.next)
+            self.assertRaises(ValueError, next, records)
             handle.close()
 
     def check_general_fails(self, filename, good_count):
-        handle = open(filename, "rU")
+        handle = open(filename, _universal_read_mode)
         tuples = QualityIO.FastqGeneralIterator(handle)
         for i in range(good_count):
-            title, seq, qual = tuples.next()  # Make sure no errors!
-        self.assertRaises(ValueError, tuples.next)
+            title, seq, qual = next(tuples)  # Make sure no errors!
+        self.assertRaises(ValueError, next, tuples)
         handle.close()
 
     def check_general_passes(self, filename, record_count):
-        handle = open(filename, "rU")
+        handle = open(filename, _universal_read_mode)
         tuples = QualityIO.FastqGeneralIterator(handle)
         #This "raw" parser doesn't check the ASCII characters which means
         #certain invalid FASTQ files will get parsed without errors.
@@ -195,9 +193,9 @@ tests = [("diff_ids", 2),
          ("trunc_at_plus", 4),
          ("trunc_at_qual", 4)]
 for base_name, good_count in tests:
-    def funct(name,c):
-        f = lambda x : x.check_all_fail("Quality/error_%s.fastq" % name,c)
-        f.__doc__ = "Reject FASTQ with %s" % name.replace("_"," ")
+    def funct(name, c):
+        f = lambda x : x.check_all_fail("Quality/error_%s.fastq" % name, c)
+        f.__doc__ = "Reject FASTQ with %s" % name.replace("_", " ")
         return f
     setattr(TestFastqErrors, "test_%s" % (base_name),
             funct(base_name, good_count))
@@ -213,9 +211,9 @@ tests = [("del", 3, 5),
          ("tab", 4, 5),
          ("null", 0, 5)]
 for base_name, good_count, full_count in tests:
-    def funct(name,c1,c2):
-        f = lambda x : x.check_qual_char("Quality/error_qual_%s.fastq"%name,c1,c2)
-        f.__doc__ = "Reject FASTQ with %s in quality" % name.replace("_"," ")
+    def funct(name, c1, c2):
+        f = lambda x : x.check_qual_char("Quality/error_qual_%s.fastq"%name, c1, c2)
+        f.__doc__ = "Reject FASTQ with %s in quality" % name.replace("_", " ")
         return f
     setattr(TestFastqErrors, "test_qual_%s" % (base_name),
             funct(base_name, good_count, full_count))
@@ -279,28 +277,30 @@ class TestReferenceFastqConversions(unittest.TestCase):
     """Tests where we have reference output."""
     def simple_check(self, base_name, in_variant):
         for out_variant in ["sanger", "solexa", "illumina"]:
-            if out_variant != "sanger":
-                #Ignore data loss warnings from max qualities
-                warnings.simplefilter('ignore', BiopythonWarning)
             in_filename = "Quality/%s_original_%s.fastq" \
                           % (base_name, in_variant)
             self.assertTrue(os.path.isfile(in_filename))
-            #Load the reference output...
-            with open("Quality/%s_as_%s.fastq"
-                      % (base_name, out_variant), "rU") as handle:
+            # Load the reference output...
+            with open("Quality/%s_as_%s.fastq" % (base_name, out_variant),
+                      _universal_read_mode) as handle:
                 expected = handle.read()
-            #Check matches using convert...
-            handle = StringIO()
-            SeqIO.convert(in_filename, "fastq-"+in_variant,
-                          handle, "fastq-"+out_variant)
-            self.assertEqual(expected, handle.getvalue())
-            #Check matches using parse/write
-            handle = StringIO()
-            SeqIO.write(SeqIO.parse(in_filename, "fastq-"+in_variant),
-                        handle, "fastq-"+out_variant)
-            self.assertEqual(expected, handle.getvalue())
-            if out_variant != "sanger":
-                warnings.filters.pop()
+
+            with warnings.catch_warnings():
+                if out_variant != "sanger":
+                    # Ignore data loss warnings from max qualities
+                    warnings.simplefilter("ignore", BiopythonWarning)
+                    warnings.simplefilter("ignore", UserWarning)
+                # Check matches using convert...
+                handle = StringIO()
+                SeqIO.convert(in_filename, "fastq-"+in_variant,
+                              handle, "fastq-"+out_variant)
+                self.assertEqual(expected, handle.getvalue())
+                # Check matches using parse/write
+                handle = StringIO()
+                SeqIO.write(SeqIO.parse(in_filename, "fastq-"+in_variant),
+                            handle, "fastq-"+out_variant)
+                self.assertEqual(expected, handle.getvalue())
+
 
 #Now add methods at run time...
 tests = [("illumina_full_range", "illumina"),
@@ -313,8 +313,8 @@ tests = [("illumina_full_range", "illumina"),
 for base_name, variant in tests:
     assert variant in ["sanger", "solexa", "illumina"]
 
-    def funct(bn,var):
-        f = lambda x : x.simple_check(bn,var)
+    def funct(bn, var):
+        f = lambda x : x.simple_check(bn, var)
         f.__doc__ = "Reference conversions of %s file %s" % (var, bn)
         return f
 
@@ -329,7 +329,7 @@ class TestQual(unittest.TestCase):
         """Check FASTQ parsing matches FASTA+QUAL parsing"""
         with open("Quality/example.fasta") as f:
             with open("Quality/example.qual") as q:
-                records1 = list(QualityIO.PairedFastaQualIterator(f,q ))
+                records1 = list(QualityIO.PairedFastaQualIterator(f, q ))
         records2 = list(SeqIO.parse("Quality/example.fastq", "fastq"))
         self.assertTrue(compare_records(records1, records2))
 
@@ -343,7 +343,7 @@ class TestQual(unittest.TestCase):
     def test_qual_out(self):
         """Check FASTQ to QUAL output"""
         records = SeqIO.parse("Quality/example.fastq", "fastq")
-        h = StringIO("")
+        h = StringIO()
         SeqIO.write(records, h, "qual")
         with open("Quality/example.qual") as expected:
             self.assertEqual(h.getvalue(), expected.read())
@@ -357,7 +357,7 @@ class TestQual(unittest.TestCase):
     def test_fasta_out(self):
         """Check FASTQ to FASTA output"""
         records = SeqIO.parse("Quality/example.fastq", "fastq")
-        h = StringIO("")
+        h = StringIO()
         SeqIO.write(records, h, "fasta")
         with open("Quality/example.fasta") as expected:
             self.assertEqual(h.getvalue(), expected.read())
@@ -374,7 +374,9 @@ class TestQual(unittest.TestCase):
 16 22 -1 -1 -1 33 -1 -1 30 27 -1 27 28 32 -1 29 -1 -1 -1 27 -1 18 9 6 -1 -1 23 16 -1 26 -1 5 7 -1 22 7 -1 18 14 8 -1 8 -1 -1 -1 11 -1 -1 4 24"""
         h = StringIO(data)
         h2 = StringIO()
-        self.assertEqual(4, SeqIO.convert(h, "qual", h2, "fastq"))
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore", BiopythonParserWarning)
+            self.assertEqual(4, SeqIO.convert(h, "qual", h2, "fastq"))
         self.assertEqual(h2.getvalue(), """@1117_10_107_F3
 ??????????????????????????????????????????????????
 +
@@ -400,7 +402,7 @@ class TestReadWrite(unittest.TestCase):
         """Read and write back simple example with upper case 2000bp read"""
         data = "@%s\n%s\n+\n%s\n" \
                % ("id descr goes here", "ACGT"*500, "!@a~"*500)
-        handle = StringIO("")
+        handle = StringIO()
         self.assertEqual(1, SeqIO.write(SeqIO.parse(StringIO(data), "fastq"), handle, "fastq"))
         self.assertEqual(data, handle.getvalue())
 
@@ -408,7 +410,7 @@ class TestReadWrite(unittest.TestCase):
         """Read and write back simple example with mixed case 1000bp read"""
         data = "@%s\n%s\n+\n%s\n" \
                % ("id descr goes here", "ACGTNncgta"*100, "abcd!!efgh"*100)
-        handle = StringIO("")
+        handle = StringIO()
         self.assertEqual(1, SeqIO.write(SeqIO.parse(StringIO(data), "fastq"), handle, "fastq"))
         self.assertEqual(data, handle.getvalue())
 
@@ -419,7 +421,7 @@ class TestReadWrite(unittest.TestCase):
                % ("id descr goes here",
                   ambiguous_dna_letters.upper(),
                   "".join(chr(33+q) for q in range(len(ambiguous_dna_letters))))
-        handle = StringIO("")
+        handle = StringIO()
         self.assertEqual(1, SeqIO.write(SeqIO.parse(StringIO(data), "fastq"), handle, "fastq"))
         self.assertEqual(data, handle.getvalue())
         #Now in lower case...
@@ -427,7 +429,7 @@ class TestReadWrite(unittest.TestCase):
                % ("id descr goes here",
                   ambiguous_dna_letters.lower(),
                   "".join(chr(33+q) for q in range(len(ambiguous_dna_letters))))
-        handle = StringIO("")
+        handle = StringIO()
         self.assertEqual(1, SeqIO.write(SeqIO.parse(StringIO(data), "fastq"), handle, "fastq"))
         self.assertEqual(data, handle.getvalue())
 
@@ -438,7 +440,7 @@ class TestReadWrite(unittest.TestCase):
                % ("id descr goes here",
                   ambiguous_rna_letters.upper(),
                   "".join(chr(33+q) for q in range(len(ambiguous_rna_letters))))
-        handle = StringIO("")
+        handle = StringIO()
         self.assertEqual(1, SeqIO.write(SeqIO.parse(StringIO(data), "fastq"), handle, "fastq"))
         self.assertEqual(data, handle.getvalue())
         #Now in lower case...
@@ -446,7 +448,7 @@ class TestReadWrite(unittest.TestCase):
                % ("id descr goes here",
                   ambiguous_rna_letters.lower(),
                   "".join(chr(33+q) for q in range(len(ambiguous_rna_letters))))
-        handle = StringIO("")
+        handle = StringIO()
         self.assertEqual(1, SeqIO.write(SeqIO.parse(StringIO(data), "fastq"), handle, "fastq"))
         self.assertEqual(data, handle.getvalue())
 
@@ -456,34 +458,33 @@ class TestWriteRead(unittest.TestCase):
     def test_generated(self):
         """Write and read back odd SeqRecord objects"""
         record1 = SeqRecord(Seq("ACGT"*500, generic_dna),  id="Test", description="Long "*500,
-                           letter_annotations={"phred_quality":[40,30,20,10]*500})
+                           letter_annotations={"phred_quality":[40, 30, 20, 10]*500})
         record2 = SeqRecord(MutableSeq("NGGC"*1000),  id="Mut", description="very "*1000+"long",
-                           letter_annotations={"phred_quality":[0,5,5,10]*1000})
-        record3 = SeqRecord(UnknownSeq(2000,character="N"),  id="Unk", description="l"+("o"*1000)+"ng",
-                           letter_annotations={"phred_quality":[0,1]*1000})
+                           letter_annotations={"phred_quality":[0, 5, 5, 10]*1000})
+        record3 = SeqRecord(UnknownSeq(2000, character="N"),  id="Unk", description="l"+("o"*1000)+"ng",
+                           letter_annotations={"phred_quality":[0, 1]*1000})
         record4 = SeqRecord(Seq("ACGT"*500),  id="no_descr", description="", name="",
-                           letter_annotations={"phred_quality":[40,50,60,62]*500})
-        record5 = SeqRecord(Seq("",generic_dna),  id="empty_p", description="(could have been trimmed lots)",
+                           letter_annotations={"phred_quality":[40, 50, 60, 62]*500})
+        record5 = SeqRecord(Seq("", generic_dna),  id="empty_p", description="(could have been trimmed lots)",
                            letter_annotations={"phred_quality":[]})
         record6 = SeqRecord(Seq(""),  id="empty_s", description="(could have been trimmed lots)",
                            letter_annotations={"solexa_quality":[]})
         record7 = SeqRecord(Seq("ACNN"*500),  id="Test_Sol", description="Long "*500,
-                           letter_annotations={"solexa_quality":[40,30,0,-5]*500})
+                           letter_annotations={"solexa_quality":[40, 30, 0, -5]*500})
         record8 = SeqRecord(Seq("ACGT"),  id="HighQual", description="With very large qualities that even Sanger FASTQ can't hold!",
-                           letter_annotations={"solexa_quality":[0,10,100,1000]})
+                           letter_annotations={"solexa_quality":[0, 10, 100, 1000]})
         #TODO - Record with no identifier?
         records = [record1, record2, record3, record4, record5, record6, record7, record8]
-        #TODO - Have a Biopython defined "DataLossWarning?"
-        warnings.simplefilter('ignore', BiopythonWarning)
-        #TODO - Include phd output?
         for format in ["fasta", "fastq", "fastq-solexa", "fastq-illumina", "qual"]:
             handle = StringIO()
-            SeqIO.write(records, handle, format)
+            with warnings.catch_warnings():
+                #TODO - Have a Biopython defined "DataLossWarning?"
+                warnings.simplefilter('ignore', BiopythonWarning)
+                SeqIO.write(records, handle, format)
             handle.seek(0)
             compare_records(records,
                             list(SeqIO.parse(handle, format)),
                             truncation_expected(format))
-        warnings.filters.pop()
 
     def check(self, filename, format, out_formats):
         for f in out_formats:
@@ -499,12 +500,8 @@ class TestWriteRead(unittest.TestCase):
         """Write and read back sanger_93.fastq"""
         self.check(os.path.join("Quality", "sanger_93.fastq"), "fastq",
                    ["fastq", "fastq-sanger", "fasta", "qual", "phd"])
-        #TODO - Have a Biopython defined "DataLossWarning?"
-        #TODO - On Python 2.6+ we can check this warning is really triggered
-        warnings.simplefilter('ignore', BiopythonWarning)
         self.check(os.path.join("Quality", "sanger_93.fastq"), "fastq",
-                   ["fastq-solexa","fastq-illumina"])
-        warnings.filters.pop()
+                   ["fastq-solexa", "fastq-illumina"])
 
     def test_sanger_faked(self):
         """Write and read back sanger_faked.fastq"""
@@ -621,7 +618,7 @@ class MappingTests(unittest.TestCase):
         self.assertEqual(6, round(QualityIO.solexa_quality_from_phred(7)))
         self.assertEqual(7, round(QualityIO.solexa_quality_from_phred(8)))
         self.assertEqual(8, round(QualityIO.solexa_quality_from_phred(9)))
-        for i in range(10,100):
+        for i in range(10, 100):
             self.assertEqual(i, round(QualityIO.solexa_quality_from_phred(i)))
 
     def test_phred_quality_from_solexa(self):
@@ -641,7 +638,7 @@ class MappingTests(unittest.TestCase):
         self.assertEqual(8, round(QualityIO.phred_quality_from_solexa(7)))
         self.assertEqual(9, round(QualityIO.phred_quality_from_solexa(8)))
         self.assertEqual(10, round(QualityIO.phred_quality_from_solexa(9)))
-        for i in range(10,100):
+        for i in range(10, 100):
             self.assertEqual(i, round(QualityIO.phred_quality_from_solexa(i)))
 
     def test_sanger_to_solexa(self):
@@ -650,17 +647,16 @@ class MappingTests(unittest.TestCase):
         #solexa_quality_from_phred function directly. For speed it uses a
         #cached dictionary of the mappings.
         seq = "N"*94
-        qual = "".join(chr(33+q) for q in range(0,94))
-        expected_sol = [min(62,int(round(QualityIO.solexa_quality_from_phred(q))))
-                        for q in range(0,94)]
-        in_handle = StringIO("@Test\n%s\n+\n%s" % (seq,qual))
-        out_handle = StringIO("")
-        #Want to ignore the data loss warning
-        #(on Python 2.6 we could check for it!)
-        warnings.simplefilter('ignore', BiopythonWarning)
-        SeqIO.write(SeqIO.parse(in_handle, "fastq-sanger"),
-                    out_handle, "fastq-solexa")
-        warnings.filters.pop()
+        qual = "".join(chr(33+q) for q in range(0, 94))
+        expected_sol = [min(62, int(round(QualityIO.solexa_quality_from_phred(q))))
+                        for q in range(0, 94)]
+        in_handle = StringIO("@Test\n%s\n+\n%s" % (seq, qual))
+        out_handle = StringIO()
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always", BiopythonWarning)
+            SeqIO.write(SeqIO.parse(in_handle, "fastq-sanger"),
+                        out_handle, "fastq-solexa")
+            self.assertTrue(len(w) <= 1, w)
         out_handle.seek(0)
         record = SeqIO.read(out_handle, "fastq-solexa")
         self.assertEqual(str(record.seq), seq)
@@ -673,17 +669,13 @@ class MappingTests(unittest.TestCase):
         #solexa_quality_from_phred function directly. For speed it uses a
         #cached dictionary of the mappings.
         seq = "N"*68
-        qual = "".join(chr(64+q) for q in range(-5,63))
+        qual = "".join(chr(64+q) for q in range(-5, 63))
         expected_phred = [round(QualityIO.phred_quality_from_solexa(q))
-                          for q in range(-5,63)]
-        in_handle = StringIO("@Test\n%s\n+\n%s" % (seq,qual))
-        out_handle = StringIO("")
-        #Want to ignore the data loss warning
-        #(on Python 2.6 we could check for it!)
-        warnings.simplefilter('ignore', BiopythonWarning)
+                          for q in range(-5, 63)]
+        in_handle = StringIO("@Test\n%s\n+\n%s" % (seq, qual))
+        out_handle = StringIO()
         SeqIO.write(SeqIO.parse(in_handle, "fastq-solexa"),
                     out_handle, "fastq-sanger")
-        warnings.filters.pop()
         out_handle.seek(0)
         record = SeqIO.read(out_handle, "fastq-sanger")
         self.assertEqual(str(record.seq), seq)
@@ -693,16 +685,15 @@ class MappingTests(unittest.TestCase):
     def test_sanger_to_illumina(self):
         """Mapping check for FASTQ Sanger (0 to 93) to Illumina (0 to 62)"""
         seq = "N"*94
-        qual = "".join(chr(33+q) for q in range(0,94))
-        expected_phred = [min(62,q) for q in range(0,94)]
-        in_handle = StringIO("@Test\n%s\n+\n%s" % (seq,qual))
-        out_handle = StringIO("")
-        #Want to ignore the data loss warning
-        #(on Python 2.6 we could check for it!)
-        warnings.simplefilter('ignore', BiopythonWarning)
-        SeqIO.write(SeqIO.parse(in_handle, "fastq-sanger"),
-                    out_handle, "fastq-illumina")
-        warnings.filters.pop()
+        qual = "".join(chr(33+q) for q in range(0, 94))
+        expected_phred = [min(62, q) for q in range(0, 94)]
+        in_handle = StringIO("@Test\n%s\n+\n%s" % (seq, qual))
+        out_handle = StringIO()
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always", BiopythonWarning)
+            SeqIO.write(SeqIO.parse(in_handle, "fastq-sanger"),
+                        out_handle, "fastq-illumina")
+            self.assertTrue(len(w) <= 1, w)
         out_handle.seek(0)
         record = SeqIO.read(out_handle, "fastq-illumina")
         self.assertEqual(str(record.seq), seq)
@@ -712,10 +703,10 @@ class MappingTests(unittest.TestCase):
     def test_illumina_to_sanger(self):
         """Mapping check for FASTQ Illumina (0 to 62) to Sanger (0 to 62)"""
         seq = "N"*63
-        qual = "".join(chr(64+q) for q in range(0,63))
-        expected_phred = range(63)
-        in_handle = StringIO("@Test\n%s\n+\n%s" % (seq,qual))
-        out_handle = StringIO("")
+        qual = "".join(chr(64+q) for q in range(0, 63))
+        expected_phred = list(range(63))
+        in_handle = StringIO("@Test\n%s\n+\n%s" % (seq, qual))
+        out_handle = StringIO()
         SeqIO.write(SeqIO.parse(in_handle, "fastq-illumina"),
                     out_handle, "fastq-sanger")
         out_handle.seek(0)
@@ -725,6 +716,55 @@ class MappingTests(unittest.TestCase):
                          expected_phred)
 
 
+class TestSFF(unittest.TestCase):
+    """Test SFF specific details."""
+    def test_overlapping_clip(self):
+        with open("Roche/greek.sff", "rb") as handle:
+            record = next(SeqIO.parse(handle, "sff"))
+        self.assertEqual(len(record), 395)
+        s = str(record.seq.lower())
+        # Apply overlapping clipping
+        record.annotations['clip_qual_left'] = 51
+        record.annotations['clip_qual_right'] = 44
+        record.annotations['clip_adapter_left'] = 50
+        record.annotations['clip_adapter_right'] = 75
+        self.assertEqual(len(record), 395)
+        self.assertEqual(len(record.seq), 395)
+        # Save the clipped record...
+        h = BytesIO()
+        count = SeqIO.write(record, h, "sff")
+        # Now reload it...
+        h.seek(0)
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always", BiopythonParserWarning)
+            record = SeqIO.read(h, "sff")
+            self.assertEqual(len(w), 1, w)
+        self.assertEqual(record.annotations['clip_qual_left'], 51)
+        self.assertEqual(record.annotations['clip_qual_right'], 44)
+        self.assertEqual(record.annotations['clip_adapter_left'], 50)
+        self.assertEqual(record.annotations['clip_adapter_right'], 75)
+        self.assertEqual(len(record), 395)
+        self.assertEqual(s, str(record.seq.lower()))
+        # And check with trimming applied...
+        h.seek(0)
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always", BiopythonParserWarning)
+            record = SeqIO.read(h, "sff-trim")
+            self.assertEqual(len(w), 1, w)
+        self.assertEqual(len(record), 0)
+
+    def test_negative_clip(self):
+        for clip in ["clip_qual_left", "clip_qual_right",
+                     "clip_adapter_left", "clip_adapter_right"]:
+            with open("Roche/greek.sff", "rb") as handle:
+                record = next(SeqIO.parse(handle, "sff"))
+            self.assertEqual(len(record), 395)
+            self.assertTrue(0 <= record.annotations[clip], record.annotations[clip])
+            record.annotations[clip] = -1
+            with BytesIO() as h:
+                self.assertRaises(ValueError, SeqIO.write, record, h, "sff")
+
+
 if __name__ == "__main__":
     runner = unittest.TextTestRunner(verbosity = 2)
     unittest.main(testRunner=runner)
